@@ -1,4 +1,7 @@
 import json
+
+import pydash as py_
+from tqdm import tqdm
 from fsm_pull.protobuf.records_pb2 import (
     Call,
     CallDataFrame,
@@ -149,47 +152,63 @@ def is_plute_style(prediction):
 def prediction_from_plute(prediction):
     try:
         intent = prediction["graph"]["output"][0][0]
+        entities_from_all_slots = []
         prediction = {
             "name": intent.get("name"),
             "score": intent.get("score"),
-            "slot": [{
+            "slot": [],
+        }
+
+        for slot in intent.get("slots", []):
+            entities = [{
+                "text": entity.get("body"),
+                "type": entity.get("type"),
+                "value": entity.get("value"),
+                "score": entity.get("score", 0),
+            } for entity in slot.get("values", [])]
+
+            prediction["slot"].append({
                 "name": slot.get("name"),
                 "type": slot.get("type"),
-                "value": [{
-                    "body": entity.get("body"),
-                    "type": entity.get("type"),
-                    "alternative_id": entity.get("alternative_id"),
-                    "value": entity.get("value"),
-                    "range": entity.get("range", {})
-                } for entity in slot.get("values", [])]
-            } for slot in intent.get("slots", [])],
-        }
-        return json.dumps(prediction)
+                "value": entities
+            })
+
+            entities_from_all_slots.append(entities)
+
+        return json.dumps(py_.flatten(entities_from_all_slots)), json.dumps(prediction)
     except (KeyError, IndexError):
-        return None
+        return json.dumps([]), None
 
 
 def prediction_from_dialogy(prediction):
     try:
-        intent = prediction["intents"][0]
+        intent = prediction.get("intents")[0]
         prediction = {
             "name": intent.get("name"),
             "score": intent.get("score"),
-            "slot": [{
+            "slot": []
+        }
+        entities_from_all_slots = []
+
+        for slot in intent.get("slots", []):
+            entities = [{
+                "text": entity.get("body"),
+                "type": entity.get("type"),
+                "value": entity.get("value"),
+                "score": entity.get("score", 0),
+            } for entity in slot.get("values", [])]
+
+            prediction["slot"].append({
                 "name": slot.get("name"),
                 "type": slot.get("type"),
-                "value": [{
-                    "body": entity.get("body"),
-                    "type": entity.get("type"),
-                    "alternative_id": entity.get("alternative_id"),
-                    "value": entity.get("value"),
-                    "range": entity.get("range", {})
-                } for entity in slot.get("values", [])]
-            } for slot in intent.get("slots", [])],
-        }
-        return json.dumps(prediction)
+                "value": entities
+            })
+
+            entities_from_all_slots.append(entities)
+
+        return json.dumps(py_.flatten(entities_from_all_slots)), json.dumps(prediction)
     except (KeyError, IndexError):
-        return None
+        return json.dumps([]), None
 
 
 def get_prediction(prediction):
@@ -200,7 +219,7 @@ def get_prediction(prediction):
         else:
             return prediction_from_dialogy(prediction)
     except (json.JSONDecodeError, TypeError):
-        return None
+        return None, None
 
 
 def build_turns_dataframe(calls_dict):
@@ -208,10 +227,12 @@ def build_turns_dataframe(calls_dict):
     Builds a dataframe from a list of Call objects.
     """
     calls = []
-    turns = []
+    flat_turns = []
     for call_dict in calls_dict:
+        turns = []
         turn = call_dict.get("turns", {})
-        for conversation in turn.get("conversations", []):
+        for conversation in tqdm(turn.get("conversations", [])):
+            entities, prediction = get_prediction(conversation.get("prediction"))
             turn_ = {
                 "id": turn.get("uuid"),
                 "created_at": turn.get("created_at"),
@@ -227,8 +248,10 @@ def build_turns_dataframe(calls_dict):
                 "asr_context": conversation.get("asr_context"),
                 "asr_provider": conversation.get("asr_provider"),
                 "language": turn.get("language_code"),
-                "prediction": get_prediction(conversation.get("prediction")),
+                "prediction": prediction,
+                "entities": entities
             }
+            flat_turns.append(turn_)
             turns.append(turn_)
         call = {
             "id": call_dict.get("uuid"),
@@ -240,4 +263,4 @@ def build_turns_dataframe(calls_dict):
             "turns": turns
         }
         calls.append(call)
-    return calls, turns
+    return calls, flat_turns
